@@ -17,12 +17,13 @@ __author__ = 'HPE'
 from base64 import b64decode
 import json
 import re
+import subprocess
+import tempfile
 
 from OpenSSL.crypto import FILETYPE_ASN1
 from OpenSSL.crypto import load_certificate
 from six.moves.urllib import parse
 import sushy
-from sushy.resources.system import mappings as sushy_map
 from sushy import utils
 
 from proliantutils import exception
@@ -124,6 +125,15 @@ _CERTIFICATE_PATTERN = (
 
 LOG = log.get_logger(__name__)
 
+# Copied from ironic/drivers/modules/redfish/inspect.py
+CPU_ARCH_MAP = {
+    sushy.PROCESSOR_ARCH_x86: 'x86_64',
+    sushy.PROCESSOR_ARCH_IA_64: 'ia64',
+    sushy.PROCESSOR_ARCH_ARM: 'arm',
+    sushy.PROCESSOR_ARCH_MIPS: 'mips',
+    sushy.PROCESSOR_ARCH_OEM: 'oem'
+}
+
 
 class RedfishOperations(operations.IloOperations):
     """Operations supported on redfish based hardware.
@@ -867,6 +877,7 @@ class RedfishOperations(operations.IloOperations):
         :param ignore : A boolean param, True when Secure_boot needs to be
                ignored. If passed False, Secure_boot security param will
                not be ignored. If nothing passed default will be False.
+        :raises: IloError, on an error from iLO.
         """
         try:
             self._update_security_parameter(sec_param="Secure Boot",
@@ -879,6 +890,128 @@ class RedfishOperations(operations.IloOperations):
             LOG.debug(msg)
             raise exception.IloError(msg)
 
+    def create_csr(self, csr_params):
+        """Creates the Certificate Signing Request.
+
+        :param csr_params: A dictionary containing all the necessary
+               information required to create CSR.
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_man = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        try:
+            cert_request = (
+                sushy_man.securityservice.https_certificate_uri.generate_csr(
+                    csr_params))
+
+            (fd, temp_file) = tempfile.mkstemp(suffix='.csr')
+
+            with open(temp_file, 'w') as f:
+                f.write(cert_request)
+
+            return temp_file
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to create the '
+                          'certificate signing request. '
+                          'Error %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def add_ssl_certificate(self, csr_params, signed_cert,
+                            private_key, pass_phrase):
+        """Creates CSR and adds the signed SSL certificate to the iLO.
+
+        :param csr_params: A dictionary containing all the necessary
+               information required to create CSR.
+        :param signed_cert: signed certificate which will be used
+               to sign the created CSR.
+        :param private_key: private key.
+        :param pass_phrase: Pass phrase for the private key.
+        :raises: IloError, on an error from iLO.
+        """
+        csr_file = self.create_csr(csr_params)
+
+        (fd, temp_file) = tempfile.mkstemp(suffix='.ext')
+        (fd, https_cert_file) = tempfile.mkstemp(suffix='.crt')
+        (fd, ss_cert_file) = tempfile.mkstemp(suffix='.crt')
+
+        with open(signed_cert, 'r') as f:
+            data = json.dumps(f.read())
+        p = re.sub(r"\"", "", data)
+        q = re.sub(r"\\n", "\r\n", p).rstrip()
+
+        c_list = re.findall(_CERTIFICATE_PATTERN, q, re.DOTALL)
+
+        if len(c_list) == 0:
+            msg = (self._("No valid certificate in %(cert_file)s.") %
+                   {"cert_file": signed_cert})
+            LOG.debug(msg)
+            raise exception.InvalidParameterValueError(msg)
+
+        ss_cert = c_list[0]
+
+        with open(ss_cert_file, 'w') as f:
+            f.write(ss_cert)
+
+        content = [
+            "authorityKeyIdentifier = keyid,issuer\n",
+            "basicConstraints = CA:true,pathlen:1\n",
+            "keyUsage = digitalSignature,keyEncipherment,keyCertSign,cRLSign",
+            "\nextendedKeyUsage = clientAuth,serverAuth\n",
+            "subjectKeyIdentifier = hash"]
+
+        with open(temp_file, 'w') as f:
+            f.writelines(content)
+
+        cert_cmd = (
+            "openssl x509 -req -days 365 -in %(csr_file)s -extfile"
+            " %(tempfile)s -CA %(cert)s -CAkey %(p_key)s -passin "
+            " pass:%(pphrase)s -CAcreateserial -out %(cert_file)s"
+            % {'csr_file': csr_file, 'tempfile': temp_file,
+               'cert': ss_cert_file, 'p_key': private_key,
+               'pphrase': pass_phrase, 'cert_file': https_cert_file})
+        try:
+            process = subprocess.Popen(cert_cmd, stdout=subprocess.PIPE,
+                                       stderr=subprocess.PIPE, shell=True)
+            out, err = process.communicate()
+        except Exception as e:
+            msg = (self._("Failed to create HTTPS certificate. "
+                          "error: %(err)s") % {"err": e})
+            LOG.debug(msg)
+            raise exception.CertificateCreationError(msg)
+
+        self.add_https_certificate(https_cert_file)
+
+    def add_https_certificate(self, cert_file):
+        """Adds the signed https certificate to the iLO.
+
+        :param certificate: Signed HTTPS certificate file.
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_man = self._get_sushy_manager(PROLIANT_MANAGER_ID)
+        try:
+            with open(cert_file, 'r') as f:
+                data = json.dumps(f.read())
+            p = re.sub(r"\"", "", data)
+            q = re.sub(r"\\n", "\r\n", p).rstrip()
+
+            c_list = re.findall(_CERTIFICATE_PATTERN, q, re.DOTALL)
+
+            if len(c_list) == 0:
+                msg = (self._("No valid certificate in %(cert_file)s.") %
+                       {"cert_file": cert_file})
+                LOG.debug(msg)
+                raise exception.IloError(msg)
+
+            cert = c_list[0]
+            sushy_man.securityservice.https_certificate_uri.import_certificate(
+                cert)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The Redfish controller failed to import the '
+                          'given certificate. '
+                          'Error: %(error)s') % {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
     def get_security_dashboard_values(self):
         """Gets all the parameters related to security dashboard.
 
@@ -1174,8 +1307,8 @@ class RedfishOperations(operations.IloOperations):
             # local_gb = sushy_system.storage_summary
             prop = {'memory_mb': (sushy_system.memory_summary.size_gib * 1024),
                     'cpus': sushy_system.processors.summary.count,
-                    'cpu_arch': sushy_map.PROCESSOR_ARCH_VALUE_MAP_REV.get(
-                    sushy_system.processors.summary.architecture),
+                    'cpu_arch': CPU_ARCH_MAP.get(
+                        sushy_system.processors.summary.architecture),
                     'local_gb': common_storage.get_local_gb(sushy_system)}
             return {'properties': prop,
                     'macs': sushy_system.ethernet_interfaces.summary}
@@ -1696,10 +1829,55 @@ class RedfishOperations(operations.IloOperations):
             msg = 'TLS certificate cannot be upload in BIOS boot mode'
             raise exception.IloCommandNotSupportedInBiosError(msg)
 
-    def remove_tls_certificate(self, cert_file_list=[]):
-        """Removes the TLS certificate from the iLO.
+    def _get_fps_from_file(self, cert_file):
+        """Gets the finger prints from the certificate file.
 
-        :param cert_file_list: List of TLS certificate files
+        Parse the file passed in to get the certificates. For each certificate,
+        find the fingerprint by calculating the digest of base64 decoded
+        content. Finally return the list of the fingerprints.
+
+        :param cert_file: TLS certificate file containing one or more
+               certificates.
+
+        :return: Returns the list of FPs for the certificates in the file.
+        """
+        fp_list = []
+        with open(cert_file, 'r') as f:
+            data = json.dumps(f.read())
+            p = re.sub(r"\"", "", data)
+            q = re.sub(r"\\n", "\r\n", p).rstrip()
+
+            c_list = re.findall(_CERTIFICATE_PATTERN, q, re.DOTALL)
+
+            if len(c_list) == 0:
+                LOG.warning("Could not find any valid certificate in "
+                            "%(cert_file)s. Ignoring." %
+                            {"cert_file": cert_file})
+                return fp_list
+
+            for content in c_list:
+                pem_lines = [line.strip() for line in (
+                    content.strip().split('\n'))]
+
+                try:
+                    der_data = b64decode(''.join(pem_lines[1:-1]))
+                except ValueError:
+                    LOG.warning("Illegal base64 encountered "
+                                "in the certificate.")
+                else:
+                    cert = load_certificate(FILETYPE_ASN1, der_data)
+                    fp = cert.digest('sha256').decode('ascii')
+                    fp_list.append(fp)
+        return fp_list
+
+    def remove_tls_certificate(self, cert_file_list=[],
+                               excl_cert_file_list=[]):
+        """Removes the TLS CA certificates from the iLO.
+
+        :param cert_file_list: List of TLS CA certificate files
+        :param excl_cert_file_list: List of TLS CA certificate files to be
+               retained on the iLO. These certificates will not be
+               removed from the iLO.
 
         :raises: IloError, on an error from iLO.
         :raises: IloCommandNotSupportedError, if the command is
@@ -1713,49 +1891,42 @@ class RedfishOperations(operations.IloOperations):
 
         cert_dict = {}
         del_cert_list = []
+        exc_fp_list = []
+
+        for exc_cert_file in excl_cert_file_list:
+            efp_list = self._get_fps_from_file(exc_cert_file)
+            exc_fp_list.extend(efp_list)
+
+        LOG.debug("Excluding certificates with FingerPrints: %(exc_fp_list)s",
+                  {'exc_fp_list': exc_fp_list})
 
         if not cert_file_list:
             tls_certificates = (sushy_system.bios_settings.tls_config.
                                 tls_certificates)
             for cert in tls_certificates:
                 fp = cert.get("FingerPrint")
+
+                if fp not in exc_fp_list:
+                    cert_fp = {
+                        "FingerPrint": fp
+                    }
+                    del_cert_list.append(cert_fp)
+
+        else:
+            all_fp_list = []
+
+            for cert_file in cert_file_list:
+                afp_list = self._get_fps_from_file(cert_file)
+                all_fp_list.extend(afp_list)
+
+            final_fp_set = set(all_fp_list) - set(exc_fp_list)
+
+            for fp in final_fp_set:
                 cert_fp = {
                     "FingerPrint": fp
                 }
                 del_cert_list.append(cert_fp)
 
-        else:
-            for cert_file in cert_file_list:
-                with open(cert_file, 'r') as f:
-                    data = json.dumps(f.read())
-                    p = re.sub(r"\"", "", data)
-                    q = re.sub(r"\\n", "\r\n", p).rstrip()
-
-                    c_list = re.findall(_CERTIFICATE_PATTERN, q, re.DOTALL)
-
-                    if len(c_list) == 0:
-                        LOG.warning("Could not find any valid certificate in "
-                                    "%(cert_file)s. Ignoring." %
-                                    {"cert_file": cert_file})
-                        continue
-
-                    for content in c_list:
-                        pem_lines = [line.strip() for line in (
-                            content.strip().split('\n'))]
-
-                        try:
-                            der_data = b64decode(''.join(pem_lines[1:-1]))
-                        except ValueError:
-                            LOG.warning("Illegal base64 encountered "
-                                        "in the certificate.")
-                        else:
-                            cert = load_certificate(FILETYPE_ASN1, der_data)
-                            fp = cert.digest('sha256').decode('ascii')
-                            cert_fp = {
-                                "FingerPrint": fp
-                            }
-                            del_cert_list.append(cert_fp)
-
         if len(del_cert_list) == 0:
             msg = (self._("No valid certificate in %(cert_file_list)s.") %
                    {"cert_file_list": cert_file_list})
